#! /usr/bin/python3
# -*- coding: utf-8 -*-

'''
Tests for unity-scope-remmina
'''

from gi.repository import Unity
from unittest import TestCase
import imp


class ResultSet(Unity.ResultSet):
    def __init__(self):
        Unity.ResultSet.__init__(self)
        self.results = []

    def do_add_result(self, result):
        self.results.append({'uri': result.uri,
                             'title': result.title,
                             'comment': result.comment,
                             'icon': result.icon_hint,
                             'protocol': result.metadata['protocol'].get_string(),
                             'server': result.metadata['server'].get_string()})


class ScopeTestCase(TestCase):
    def init_scope(self, scope_path):
        self.scope_module = imp.load_source('scope', scope_path)
        self.scope = self.scope_module.load_scope()

    def perform_query(self, query, filter_set=Unity.FilterSet.new()):
        result_set = ResultSet()
        ctx = Unity.SearchContext.create(query, 0, filter_set,
                                         None, result_set, None)
        s = self.scope.create_search_for_query(ctx)
        s.run()
        return result_set


class TestRemmina(ScopeTestCase):
    def setUp(self):
        self.init_scope('src/unity_remmina_daemon.py')

    def tearDown(self):
        self.scope = None
        self.scope_module = None

    def test_valid_searches(self):
        '''
        test_valid_searches(): Tests that a valid result is returned when a valin .remmina file is present
        '''
        self.scope_module.REMMINA_SETTINGS_PATH = 'tests/data/mock_remmina_pass/'
        expected_results = ['Test Remmina Box',
                            'tests/data/mock_remmina_pass/mock_remmina_pass.remmina',
                            'VNC',
                            '192.168.1.3']
        results = []
        for search in ['test']:
            result_set = self.perform_query(search)
            results.append(result_set.results[0]['title'])
            results.append(result_set.results[0]['uri'])
            results.append(result_set.results[0]['protocol'])
            results.append(result_set.results[0]['server'])
        self.assertEqual(results, expected_results)

    def test_failing_search(self):
        '''
        test_failing_search(): Tests that 0 results are returned when only garbage data is available
        '''
        self.scope_module.REMMINA_SETTINGS_PATH = 'tests/data/mock_remmina_fail/'
        for s in ['test']:
            result_set = self.perform_query(s)
            self.assertEqual(len(result_set.results), 0)

    def test_activation(self):
        '''
        test_activation(): Tests the scope's do_activate() method
        '''
        result = Unity.ScopeResult()
        self.scope_module.REMMINA_EXECUTABLE = 'tests/data/fake_remmina.py'
        result.uri = "tests/data/mock_remmina_pass/mock_remmina_pass.remmina"
        activation = self.scope.activate(result, Unity.SearchMetadata(), None)
        self.assertEqual(activation.props.goto_uri, None)
        self.assertEqual(activation.props.handled, Unity.HandledType.HIDE_DASH)


if __name__ == '__main__':
    unittest.main()
